{#
#
# Copyright (C) 2015-2024 PyFPGA Project
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#}

if { [ catch { open_hw_manager } ] } { open_hw }
connect_hw_server
open_hw_target
puts [get_hw_devices]
set obj [lindex [get_hw_devices [current_hw_device]] 0]
set_property PROGRAM.FILE {{ bitstream }} $obj
program_hw_devices $obj
